//
//  AmityMessagesTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 5/13/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK
import AVKit
import SwiftUI

private let kContentOffsetY: CGFloat = 10
private let kContentOffsetX: CGFloat = 0

final class AmityMessagesTableViewController: UITableViewController, MessageDataSourceListener, UIDocumentInteractionControllerDelegate, AmityEditMessageViewControllerDelegate, AmityCustomViewControllerDelegate, AmityAddReactionsDelegate, AmityChatTextTableViewCellDelegate {
    
    var client: AmityClient!
    var channelId: String!
    var subChannelId: String!
    
    private var reversePreference: Bool {
        get { return UserDefaults.standard.channelReversePreference[channelId] ?? true }
        set { UserDefaults.standard.channelReversePreference[channelId] = newValue }
    }
    
    private lazy var flagManager = FlagManager(client: client, viewController: self)
    private lazy var tagManager = TagManager(client: client, viewController: self)
    
    private var dataSource: MessageDataSource?
    
    private let userRepo = AmityUserRepository(client: AmityManager.shared.client!)
    private var fetchedUser: AmityObject<AmityUser>?
    private var userToken: AmityNotificationToken?
    
    private lazy var reactionRepo = AmityReactionRepository(client: client)
    
    var messageRepository: AmityMessageRepository?
    
    let audioManager = AudioMessageHandler()
    
    /// If set, the next message created by the user will be a child of this
    /// message.
    var replyToMessageId: String?
    
    let fileRepo = AmityFileRepository(client: AmityManager.shared.client!)
    
    private func getUserRepository() -> AmityUserRepository {
        return AmityUserRepository(client: client)
    }
    
    var messageQueryOptions: AmityMessageQueryOptions?
    var messagesCollection: AmityCollection<AmityMessage>?
    var filterByParentIdActive: Bool = false
    var sortOption: AmityMessageQuerySortOptions = .lastCreated
    var parentFilter: AmityMessageParentFilter = .noParent
    var messageType: AmityMessageType?
    
    private func observeMessages() {
        messageRepository = AmityMessageRepository(client: client)
        let userDefaults: UserDefaults = .standard
        
        let includingTags: [String] = userDefaults.channelPreferenceIncludingTags[channelId] ?? []
        let excludingTags: [String] = userDefaults.channelPreferenceExcludingTags[channelId] ?? []
        
        filterByParentIdActive = userDefaults.channelPreferenceFilterByParentIdActive[channelId] ?? false
        let parentId: String? = userDefaults.channelPreferenceFilterByParentId[channelId]
        
        if let messageTypeRawValue = userDefaults.channelPreferenceFilterByType[channelId], let messageTypeRawValue = messageTypeRawValue {
            messageType = AmityMessageType(rawValue: messageTypeRawValue)
        }
        
        sortOption = reversePreference ? .lastCreated : .firstCreated
        
        if filterByParentIdActive {
            parentFilter = AmityMessageParentFilter.parent(id: parentId)
        } else {
            parentFilter = AmityMessageParentFilter.noParent
        }
        
        messageQueryOptions = AmityMessageQueryOptions(subChannelId: subChannelId, includingTags: includingTags, excludingTags: excludingTags, messageParentFilter: parentFilter, type: messageType, sortOption: sortOption)
        
        queryMessages(queryOptions: messageQueryOptions!)
    }
    
    @objc func scrollToBottom() {
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.2) {
            guard let messageCount = self.dataSource?.numberOfMessages(), messageCount > 0 else { return }
            self.tableView.layoutIfNeeded()
            let indexPath: IndexPath = IndexPath(item: messageCount - 1, section: 0)
            self.tableView.scrollToRow(at: indexPath, at: .bottom, animated: true)
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.contentInset = UIEdgeInsets(top: kContentOffsetY, left: kContentOffsetX,
                                              bottom: kContentOffsetY, right: kContentOffsetX)
        
        NotificationCenter.default.addObserver(self, selector: #selector(notificationReceived(_:)), name: NSNotification.Name.notificationClicked, object: nil)
    }
    
    @objc func notificationReceived(_ notification: Notification) {
        guard let data = notification.userInfo?["data"] as? [String : Any],
              let subChannelId = data["subChannelId"] as? String,
              let messageId = data["messageId"] as? String,
              let messageQueryOptions
        else {
            return
        }
        
        let queryOptions = AmityMessageQueryOptions(subChannelId: subChannelId, includingTags: messageQueryOptions.includingTags, excludingTags: messageQueryOptions.excludingTags, messageParentFilter: parentFilter, type: messageType, aroundMessageId: messageId ,sortOption: sortOption)
        
        queryMessages(queryOptions: queryOptions)
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        observeMessages()
        super.viewWillAppear(animated)
    }
    
    // MARK: UITableViewDataSource
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataSource?.numberOfMessages() ?? 0
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let message: AmityMessage = getMessage(for: indexPath), let cellIdentifier = cellIdentifier(for: message, client: client) else {
            return UITableViewCell()
        }
        
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifier, for: indexPath)
        (cell as? AmityChatCellProtocol)?.display(message, client: client)
        (cell as? AmityChatTextTableViewCell)?.delegate = self
        
        if message.messageType == .image {
            Log.add(info: "-- List Item -- \n")
            Log.add(info: "Message Id: \(message.messageId) | Message channel segment: \(message.channelSegment)")
            Log.add(info: "Sync State: \(message.syncState.description)")
            
            let imageInfo = message.getImageInfo()
            Log.add(info: "Image Id: \(imageInfo?.fileId)")
            Log.add(info: "Image URL: \(imageInfo?.fileURL)")
        }
        
        if message.messageType == .video {
            Log.add(info: "-- List Item -- \n")
            Log.add(info: "Message Id: \(message.messageId) | Message channel segment: \(message.channelSegment)")
            Log.add(info: "Sync State: \(message.syncState.description)")
            
            let videoInfo = message.getVideoInfo()
            Log.add(info: "Video Id: \(videoInfo?.fileId)")
            Log.add(info: "Video URL: \(videoInfo?.fileURL)")
        }
        
        return cell
    }
    
    private func getMessage(for indexPath: IndexPath) -> AmityMessage? {
        return dataSource?.message(for: indexPath)
    }
    
    private func cellIdentifier(for message: AmityMessage, client: AmityClient) -> String? {
        switch message.messageType {
        case .custom where message.userId == client.currentUserId:
            fallthrough
        case .text where message.userId == client.currentUserId:
            return "ChatTextMeCell"
        case .custom:
            return "ChatTextOtherCell"
        case .text:
            return "ChatTextOtherCell"
        case .image:
            return message.userId == client.currentUserId ? "ChatImageMeCell" : "ChatImageCell"
        case .file, .audio:
            return message.userId == client.currentUserId ? "ChatFileMeCell" : "ChatFileCell"
        case .video:
            return message.userId == client.currentUserId ? "ChatVideoMeCell" : "ChatVideoCell"
        @unknown default:
            return nil
        }
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        guard let message: AmityMessage = getMessage(for: indexPath) else { return 0 }
        
        switch message.messageType {
        case .image, .video:
            let width = message.data?["width"] as? CGFloat ?? 200
            let height =  message.data?["height"] as? CGFloat ?? 200
            let imageSize = CGSize(width: width, height: height)
            return self.height(for: imageSize, withinSize: CGSize(width: 200, height: 200)) + 14
        case .text, .custom, .file, .audio:
            fallthrough
        @unknown default:
            return UITableView.automaticDimension
            //return super.tableView(tableView, heightForRowAt: indexPath)
        }
    }
    
    private func height(for size: CGSize, withinSize newSize: CGSize) -> CGFloat {
        let aspectWidth: CGFloat = newSize.width / size.width
        let aspectHeight: CGFloat = newSize.height / size.height
        let aspectRatio: CGFloat = min(aspectWidth, aspectHeight)
        
        return size.height * aspectRatio
    }
    
    // MARK: UITableViewDelegate
    
    /// Consider a channel with the following messages:
    ///
    ///   ABC ... XYZ ⟶ last (newest) message
    ///   ↳ first (oldest) message
    ///
    /// Based on whether we load the messages in reverse or not, we need to load more
    /// messages at different times.
    /// Below we see the two possible configurations, the dots tell us what is the direction
    /// where we expect the next batch of messages to be displayed at, therefore we use this
    /// information to decide when to load based on the scrolling event.
    ///
    /// 1)                               2)
    ///   reverse = false                  reverse = true
    ///   (Chronological order)            (Non-chronological order)
    ///
    ///          +=======+                            .
    ///          |   A   |                            .
    ///          |   B   |                        +=======+
    ///          |   C   |                        |   X   |
    ///          |   .   |                        |   Y   |
    ///          |   .   |                        |   Z   |
    ///          |       |                        |       |
    ///
    override func scrollViewWillEndDragging(_ scrollView: UIScrollView,
                                            withVelocity velocity: CGPoint,
                                            targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        
        // segmentAsc - load next page when scrolled to bottom
        // segmentDec - load next page when scroleld to top
        let fullHeight = scrollView.contentSize.height
        let targetContentOffsetY = targetContentOffset.pointee.y
        
        print("TargetContentOffset ----- \(targetContentOffset.pointee.y), ScrollViewContentOffset ----- \(scrollView.contentOffset.y)")
        if targetContentOffsetY + scrollView.frame.size.height >= fullHeight {
            // scroll view reached to bottom
            reversePreference ? dataSource?.loadPrevious() : dataSource?.loadNext()
        } else if targetContentOffsetY <= 0 {
            // scroll view reached to top
            reversePreference ? dataSource?.loadNext() : dataSource?.loadPrevious()
        }
    }
    
    override func tableView(_ tableView: UITableView, trailingSwipeActionsConfigurationForRowAt indexPath: IndexPath) -> UISwipeActionsConfiguration? {
        
        let flagAction = UIContextualAction(style: .destructive, title: "Flag Options") { [weak self] action, view, completion in
            guard let message = self?.getMessage(for: indexPath) else { return }
            self?.flagManager.displayFlagAlertController(for: message)
            completion(true)
        }
        
        let tagAction = UIContextualAction(style: .normal, title: "Set Tags") { [weak self] _, _, completion in
            guard let message = self?.getMessage(for: indexPath) else { return }
            self?.tagManager.displayTagAlertController(for: message)
            completion(true)
        }
        tagAction.backgroundColor = UIColor(named: "AmityGreen")
        
        let filterByParentIdAction = UIContextualAction(style: .normal, title: "Filter Childs") { [weak self] (_, _, completion) in
            guard let channelId = self?.channelId, let message = self?.getMessage(for: indexPath) else { return }
            
            let userDefaults: UserDefaults = .standard
            userDefaults.channelPreferenceFilterByParentIdActive[channelId] = true
            userDefaults.channelPreferenceFilterByParentId[channelId] = message.messageId
            self?.observeMessages()
            
            completion(true)
        }
        filterByParentIdAction.backgroundColor = UIColor(named: "AmityOrange")
        
        let replyToAction = UIContextualAction(style: .normal, title: "Reply To") { [weak self] (_, _, completion) in
            guard let message = self?.getMessage(for: indexPath) else { return }
            self?.replyToMessageId = message.messageId
            let alertController = UIAlertController(title: "Reply to set ✅", message: "Please send a normal message to reply to this message", preferredStyle: .alert)
            
            let defaultAction = UIAlertAction(title: "OK", style: .cancel, handler: nil)
            alertController.addAction(defaultAction)
            
            self?.present(alertController, animated: true, completion: nil)
            
            completion(true)
        }
        
        let config = UISwipeActionsConfiguration(actions: [flagAction, tagAction, filterByParentIdAction, replyToAction])
        return config
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        guard let message = getMessage(for: indexPath) else { return }
        
        if message.isDeleted {
            let alert = UIAlertController(title: "Error", message: "Message has been deleted", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
            present(alert, animated: true, completion: nil)
        } else {
            showAlertSheet(message: message)
        }
    }
    
    private func showRestrictionAlert() {
        let alert = UIAlertController(title: "Error", message: "You are not authorized for editing this message", preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
        present(alert, animated: true, completion: nil)
    }
    
    private func showAlertSheet(message: AmityMessage) {
        let messageId = message.messageId
        let alert = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
        alert.addAction(UIAlertAction(title: "Delete", style: .destructive, handler: { [weak self] _ in
            self?.deleteAction(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Update", style: .default, handler: { [weak self] _ in
            self?.updateAction(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Add A Reaction", style: .default, handler: { [weak self] _ in
            self?.addReactionAction(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Remove A Reaction", style: .default, handler: { [weak self] _ in
            self?.removeReactionAction(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Show My Reaction", style: .default, handler: { [weak self] _ in
            self?.showMyReactionAction(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Show All Reaction", style: .default, handler: { [weak self] _ in
            self?.showAllReactions(forMessage: message)
        }))
        alert.addAction(UIAlertAction(title: "Delete Msg if Failed", style: .default, handler: { [weak self] _ in
            self?.deleteFailedMessage(message: message)
        }))
        alert.addAction(UIAlertAction(title: "Test Reaction", style: .default, handler: { [weak self] _ in
            self?.showReactionTestView(messageId: messageId)
        }))
        
        if let parentId = message.parentId {
            alert.addAction(UIAlertAction(title: "Jump To Parent Message", style: .default, handler: { [weak self] _ in
                guard let self, let messageQueryOptions else { return }
                
                let queryOptions = AmityMessageQueryOptions(subChannelId: messageQueryOptions.subChannelId, includingTags: messageQueryOptions.includingTags, excludingTags: messageQueryOptions.excludingTags, messageParentFilter: parentFilter, type: messageType, aroundMessageId: parentId ,sortOption: sortOption)
                
                self.queryMessages(queryOptions: queryOptions)
            }))
        }
        
        if message.messageType == .audio {
            alert.addAction(UIAlertAction(title: "Play Audio", style: .default, handler: { [weak self] _ in
                self?.downloadFileFromMessage(message: message, completion: { (audioFileURL) in
                    DispatchQueue.main.async {
                        
                        self?.audioManager.prepareAudioPlayer(url: audioFileURL)
                        self?.audioManager.playAudioRecording()
                    }
                })
            }))
        }
        
        if message.messageType == .file {
            alert.addAction(UIAlertAction(title: "Download File", style: .default, handler: { [weak self] _ in
                
                self?.downloadFileFromMessage(message: message, completion: { (fileURL) in
                    DispatchQueue.main.async {
                        
                        let dc = UIDocumentInteractionController(url: fileURL)
                        dc.delegate = self
                        dc.presentPreview(animated: true)
                    }
                })
            }))
        }
        
        if message.messageType == .video {
            alert.addAction(UIAlertAction(title: "Watch Video", style: .default, handler: { [weak self] _ in
                guard let videoInfo = message.getVideoInfo() else { return }
                self?.addVideoAlertAction(videoInfo: videoInfo)
            }))
        }
        
        alert.addAction(UIAlertAction(title: "Print Details", style: .default, handler: { action in
            
            let messageId = message.messageId
            let localMessageId = message.uniqueId
            let channelId = message.channelId
            let subChannelId = message.subChannelId
            let messageType = message.messageType.description
            let messageFlagCount = message.flagCount
            let messageData = message.data?.description ?? ""
            let createdAt = message.createdAt
            let editedAt = message.editedAt
            let isEdited = message.isEdited
            let isDeleted = message.isDeleted
            
            let details = """
                LocalMessageId: \(localMessageId),
                MessageId: \(messageId),
                ChannelId: \(channelId),
                SubChannelId: \(subChannelId),
                MessageType: \(messageType),
                Flag Count: \(messageFlagCount),
                Message Data: \(messageData),             
                Created At: \(createdAt),
                Edited At: \(editedAt),
                Is Edited: \(isEdited),
                Is Deleted: \(isDeleted),
                ReadCount: \(message.readCount)
                DeliveredCount: \(message.deliveredCount)
                Tag: \(message.tags)
                """
            
            self.showMessageDetails(details: details)

            Log.add(info: "\n---- Details ----")
            Log.add(info: "Local Message Id: \(message.uniqueId)")
            Log.add(info: "Message Id: \(message.messageId)")
            Log.add(info: "Channel Id: \(message.channelId)")
            Log.add(info: "SubChannel Id: \(message.subChannelId)")
            Log.add(info: "Sync State: \(message.syncState.description)")
            Log.add(info: "Message Type: \(message.messageType.description)")
            Log.add(info: "Message Flag Count: \(message.flagCount)")
            Log.add(info: "Message Read Count: \(message.readCount)")
            Log.add(info: "Message Read Count: \(message.deliveredCount)")
            Log.add(info: "Message Tag: \(message.tags)")
            
            Log.add(info: "Data: \(String(describing: message.data?.description))")
            Log.add(info: "----------------- \n")
            
        }))
        
        alert.addAction(UIAlertAction(title: "Copy Message Id", style: .default) { action in
            UIPasteboard.general.string = message.messageId
        })
        
        alert.addAction(UIAlertAction(title: "Real Time Event", style: .default) { [weak self] action in
            let realTimeController = MessageRealTimeEventController()
            realTimeController.manager = MessageRealTimeEventManager(message: message)

            // start observe only for auto subscription channel
            if let client = self?.client {
                let channelRepository = AmityChannelRepository(client: client)
                if let channelId = self?.channelId, let channel = channelRepository.getChannel(channelId).object {
                    if channel.channelType == .community || channel.channelType == .conversation {
                        realTimeController.shouldStartObserve = true
                    }
                }
            }
            
            self?.navigationController?.pushViewController(realTimeController, animated: true)
        })
        
        alert.addAction(UIAlertAction(title: "Mark delivered", style: .default) { [weak self] action in
            Task {
                do {
                    try await message.markAsDelivered()
                    guard let self else { return }
                    AppUtility.showAlert(in: self, title: "", message: "Mark delivered success")
                } catch let error {
                    guard let self else { return }
                    AppUtility.showAlert(in: self, title: "Error", message: "\(error.localizedDescription)")
                }
            }
        })
        
        alert.addAction(UIAlertAction(title: "Read/Deliver User list", style: .default) { [weak self] action in
            
            let viewmodel = ReadDeliverUserListViewModel(message: message)
            let view = ReadDeliverUserListView(viewModel: viewmodel)
            let hostingController = UIHostingController(rootView: view)
            self?.navigationController?.pushViewController(hostingController, animated: true)

        })
        
        if let fileId = message.fileId {
            var fileURL: String
            if message.messageType == .image {
                let data = message.getImageInfo()
                fileURL = data?.fileURL ?? ""
            } else {
                let data = message.getFileInfo()
                fileURL = data?.fileURL ?? ""
            }
            
            alert.addAction(UIAlertAction(title: "Copy File Id", style: .default) { action in
                UIPasteboard.general.string = fileId
            })
        
            alert.addAction(UIAlertAction(title: "Open File URL", style: .default) { action in
                if let url = URL(string: fileURL) {
                    UIApplication.shared.open(url)
                }
            })
            
            alert.addAction(UIAlertAction(title: "Delete File", style: .destructive) { action in
                Task {
                    let alertMessage: String
                    do {
                        let _ = try await self.fileRepo.deleteFile(fileId: fileId)
                        alertMessage = "File successfully deleted"
                    } catch {
                        alertMessage = "Error while deleting file"
                    }
                    self.showAlert(message: alertMessage)
                }
            })
        }
        
        alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
        present(alert, animated: true, completion: nil)
    }
        
    func showAlert(message: String) {
        let alert = UIAlertController(title: "", message: message, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "Dismiss", style: .cancel, handler: nil))
        present(alert, animated: true, completion: nil)
    }
    
    func addVideoAlertAction(videoInfo: AmityVideoData) {
        let resolutions: [AmityVideoResolution] = Array(videoInfo.videoUrls.keys).sorted()
        let alertController = UIAlertController(title: "Watch Video", message: nil, preferredStyle: .actionSheet)
        
        if let originalUrl = URL(string: videoInfo.fileURL) {
            let action = UIAlertAction(title: "Original [Raw]", style: .default) { [weak self] action in
                self?.playVideo(url: originalUrl)
            }
            alertController.addAction(action)
        }
        
        for resolution in resolutions {
            if let fileUrl = videoInfo.getVideo(resolution: resolution),
               let url = URL(string: fileUrl) {
                let pick = UIAlertAction(title: "\(resolution.rawValue)", style: .default) { [weak self] action in
                    self?.playVideo(url: url)
                }
                alertController.addAction(pick)
            }
        }
        
        let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancel)
        present(alertController, animated: true, completion: nil)
    }
    
    func playVideo(url: URL) {
        let player = AVPlayer(url: url)
        let controller = AVPlayerViewController()
        controller.player = player
        present(controller, animated: true) {
            player.play()
        }
    }
    
    func showMessageDetails(details: String) {
        let alertController = UIAlertController(title: "Message Details", message: details, preferredStyle: .alert)
        alertController.addAction(UIAlertAction(title: "Dismiss", style: .default, handler: nil))
        
        self.present(alertController, animated: true, completion: nil)
    }
    
    func showReactionTestView(messageId: String) {
        let viewModel = ReactionTestViewModel(messageId: messageId)
        let rootView = ReactionTestView(viewModel: viewModel)
        let hostController = UIHostingController(rootView: rootView)
        self.navigationController?.pushViewController(hostController, animated: true)
    }
        
    private func deleteFailedMessage(message: AmityMessage) {
        guard message.syncState == .error else { return }
        
        messageRepository?.deleteFailedMessage(message.messageId, completion: { (isSuccess, error) in
            Log.add(info: "Deleting Failed Message Status: \(isSuccess)")
            self.tableView.reloadData()
        })
    }
    
    private func queryMessages(queryOptions: AmityMessageQueryOptions) {
        self.dataSource = nil
        self.messagesCollection = nil
        
        self.messagesCollection = self.messageRepository!.getMessages(options: queryOptions)
        self.dataSource = MessageDataSource(messagesCollection: messagesCollection!,
                                       reverse: reversePreference)
        self.dataSource?.dataSourceObserver = self
        
        // Search the indexPath of parent message, and scroll to it
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.8) {
            for (index, message) in self.messagesCollection!.allObjects().enumerated() {
                if message.messageId == queryOptions.aroundMessageId {
                    
                    let calculatedIndex = self.reversePreference ? self.messagesCollection!.count() - index - 1 : index
                    guard 0..<self.tableView.numberOfRows(inSection: 0) ~= calculatedIndex else { break }
                    
                    let indexPath = IndexPath(row: calculatedIndex, section: 0)
                    self.tableView.scrollToRow(at: indexPath, at: .middle, animated: true)
                    
                    break
                    
                }
            }
        }
    }
    
    // MARK: - Alert Sheet Action
    private func deleteAction(forMessage message: AmityMessage) {
        let performDeleteAction: (Bool) -> Void = { [weak self] hasPermission in
            guard hasPermission else {
                self?.showRestrictionAlert()
                return
            }
            Task {
                do {
                    let result = try await self?.messageRepository?.softDeleteMessage(withId: message.messageId)
                    let alert = UIAlertController(title: nil, message: "Message succesfully deleted", preferredStyle: .alert)
                    alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                    self?.present(alert, animated: true, completion: nil)
                    self?.tableView.reloadData()
                } catch(let error) {
                    Log.add(info: "Deleting Message Status: \(error)")
                }
            }
        }
        
        // if current user is a message owner, then remove message.
        // else check if the user has a permission or not.
        if message.userId == client.currentUserId {
            performDeleteAction(true)
        } else {
            client.hasPermission(.deleteMessage, forChannel: channelId) { (hasPermission) in
                performDeleteAction(hasPermission)
            }
        }
    }
    
    private func updateAction(forMessage message: AmityMessage) {
        let performUpdateAction: (Bool) -> Void = { [weak self] hasPermission in
            guard hasPermission else {
                self?.showRestrictionAlert()
                return
            }
            if message.messageType == .text {
                guard let vc = AmityEditMessageViewController.make() as? AmityEditMessageViewController else { return }
                vc.delegate = self
                vc.client = self?.client
                vc.channelId = self?.channelId
                vc.subChannelId = self?.subChannelId
                vc.setMessage(message: message)
                let nvc = UINavigationController(rootViewController: vc)
                self?.navigationController?.present(nvc, animated: true, completion: nil)
            } else if message.messageType == .custom {
                guard let vc = AmityCustomViewController.makeViewController() as? AmityCustomViewController else { return }
                vc.setMessage(message: message)
                vc.delegate = self
                let nvc = UINavigationController(rootViewController: vc)
                self?.navigationController?.present(nvc, animated: true, completion: nil)
            } else {
                let alert = UIAlertController(title: "Error", message: "Message type not supported for editing", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                self?.present(alert, animated: true, completion: nil)
            }
        }
        
        // if current user is a message owner, then edit message.
        // else check if the user has a permission or not.
        if message.userId == client.currentUserId {
            performUpdateAction(true)
        } else {
            client.hasPermission(.editMessage, forChannel: channelId) {  (hasPermission) in
                performUpdateAction(hasPermission)
            }
        }
    }
    
    private func addReactionAction(forMessage message: AmityMessage) {
        guard let vc = AmityAddReactionsViewController.makeViewController() as? AmityAddReactionsViewController else { return }
        vc.setupView(type: .add, message: message)
        vc.delegate = self
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
    }
    
    private func removeReactionAction(forMessage message: AmityMessage) {
        guard let vc = AmityAddReactionsViewController.makeViewController() as? AmityAddReactionsViewController else { return }
        vc.setupView(type: .remove, message: message)
        vc.delegate = self
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
    }
    
    private func showMyReactionAction(forMessage message: AmityMessage) {
        let vc = AmityReactionsViewController.makeViewController()
        let viewModels: [ReactionViewModel] = message.myReactions.map { reaction in
            ReactionViewModel(reactionName: reaction, userDisplayName: "me", createdAt: nil)
        }
        let dataSource = StaticReactionDatasource(viewModels: viewModels)
        vc.setDatasource(dataSource)
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
    }
    
    private func showAllReactions(forMessage message: AmityMessage) {
        let vc = AmityReactionsViewController.makeViewController()
        let reactionsCollection = reactionRepo.getReactions(
            message.messageId,
            referenceType: .message,
            reactionName: nil
        )
        let dataSource = LiveReactionDatasource(reactionsCollection: reactionsCollection)
        vc.setDatasource(dataSource)
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
    }
    
    // MARK: UIDocumentInteractionController
    func documentInteractionControllerViewControllerForPreview(_ controller: UIDocumentInteractionController) -> UIViewController {
        return self
    }
    
    // MARK: MessageDataSourceListner
    func didUpdateDataSource(hasNewMessage: Bool, error: Error?) {
        if let error {
            showAlert(message: "\(error)")
            
            if error.isAmityErrorCode(.itemNotFound) {
                queryMessages(queryOptions: messageQueryOptions!)
            }
            
            return
        }
        
        tableView.reloadData()
        
        if hasNewMessage {
            // scroll to last row on refresh
            scrollToBottom()
        }
    }
    
    // MARK: AmityEditMessageViewControllerDelegate
    
    func amityEdit(_ viewController: AmityEditMessageViewController, willUpdateText text: String, onMessage message: AmityMessage?, metadata: [String: Any]?, mentionees: AmityMentioneesBuilder?) {
        guard let message = message else { return }
        
        let editor = AmityMessageEditor(client: client, messageId: message.messageId)
        Task { @MainActor in
            do {
                let _ = try await messageRepository?.editTextMessage(withId: message.messageId, text, metadata: metadata, mentionees: mentionees)
            } catch {}
            
            self.tableView.reloadData()
        }
    
    }
    
    // MARK: AmityCustomViewControllerDelegate
    
    func amityCustom(_ viewController: AmityCustomViewController, willSendCustomDataWithData data: [String : Any]) {
        // Intentionally left empty
    }
    
    func amityCustom(_ viewController: AmityCustomViewController, willSendVoiceMessageWithData audioFileURL: URL, fileName: String) {
        // Intentionally left empty
    }
    
    func amityCustom(_ viewController: AmityCustomViewController, willUpdateCustomDataWithData data: [String : Any], onMessage message: AmityMessage?) {
        guard let message = message else { return }
        let editor = AmityMessageEditor(client: client, messageId: message.messageId)
        editor.editCustomMessage(data) { [weak self] success, error in
            self?.tableView.reloadData()
        }
    }
    
    // MARK: AmityAddReactionsDelegate
    func didSendReaction(_ viewController: AmityAddReactionsViewController, withReactionName reaction: String, message: AmityMessage?) {
        guard let message = message else { return }
        sendReaction(with: message, reaction: reaction)
    }
    
    func didRemoveReaction(_ viewController: AmityAddReactionsViewController, withReactionName reaction: String, message: AmityMessage?) {
        guard let message = message else { return }
        removeReaction(with: message, reaction: reaction)
    }
    
    private func sendReaction(with message: AmityMessage, reaction: String) {
        Task { @MainActor in
            let title: String
            let description: String
            let success: Bool
            
            do {
                let result = try await reactionRepo.addReaction(reaction, referenceId: message.messageId, referenceType: .message)
                title = "Success"
                description = "Reaction \(reaction) added"
                success = result
            } catch {
                title = "Error"
                description = error.localizedDescription
                success = false
            }
            
            let alert = UIAlertController(title: title, message: description, preferredStyle: .alert)
            let style: UIAlertAction.Style = success ? .default : .destructive
            alert.addAction(UIAlertAction(title: "OK", style: style, handler: nil))
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    private func removeReaction(with message: AmityMessage, reaction: String) {
        Task { @MainActor in
            let title: String
            let description: String
            let success: Bool
            
            do {
                let result = try await reactionRepo.removeReaction(reaction, referenceId: message.messageId, referenceType: .message)
                title = "Success"
                description = "Reaction \(reaction) removed"
                success = result
            } catch {
                title = "Error"
                description = error.localizedDescription
                success = false
            }
            
            let alert = UIAlertController(title: title, message: description, preferredStyle: .alert)
            let style: UIAlertAction.Style = success ? .default : .destructive
            alert.addAction(UIAlertAction(title: "OK", style: style, handler: nil))
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    
    // MARK: AmityChatTextTableViewCellDelegate
    
    func chatTextDidReact(_ cell: AmityChatTextTableViewCell, withReaction reaction: String) {
        guard let indexPath = tableView.indexPath(for: cell), let message = dataSource?.message(for: indexPath) else { return }
        if message.myReactions.contains(reaction) {
            removeReaction(with: message, reaction: reaction)
        } else {
            sendReaction(with: message, reaction: reaction)
        }
    }
    
    func chatTextDidTapOnMention(_ cell: AmityChatTextTableViewCell, withType type: AmityMessageMentionType, userId: String?) {
        if type == .channel { return }
        fetchedUser = userRepo.getUser(userId!)
        userToken = fetchedUser?.observe { [weak self] (user, error) in
            switch user.dataStatus {
            case .fresh:
                self?.userToken?.invalidate()
                let userId = user.object?.userId ?? ""
                let displayName = user.object?.displayName ?? ""
                
                let alert = UIAlertController(title: "Tapped on the user who is mentioned in the message", message: "userId: \(userId) , display name: \(displayName)", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                self?.present(alert, animated: true, completion: nil)
                
            default :
                break
            }
        }
    }
    
    // Helpers
    func downloadFileFromMessage(message: AmityMessage, completion: @escaping (URL) -> Void) {
        do {
            var fileName = message.messageId
            var fileURL = ""
            if message.syncState == .synced {
                
                guard let fileInfo = message.getFileInfo() else { return }
                
                fileURL = fileInfo.fileURL
                let fileExtension = fileInfo.attributes["extension"] as? String ?? ""
                fileName += ".\(fileExtension)"
            } else {
                
                guard let fileInfo = message.getFileInfo(), let absURL = URL(string: fileInfo.fileURL) else { return }
                fileName = fileInfo.fileName
                
                let fileURL = URL(fileURLWithPath: absURL.path)
                
                completion(fileURL)
                return
            }
            
            Log.add(info: "File Name: \(fileName)")
            
            let cacheDirectory = try FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false)
            let destination = cacheDirectory.appendingPathComponent(fileName)
            
            downloadData(fileURL: fileURL, destination: destination) { (fileURL) in
                completion(fileURL)
            }
            
        } catch let err {
            Log.add(info: "File Download Error \(err)")
        }
    }
    
    func downloadData(fileURL: String, destination: URL, completion: @escaping (URL) -> Void) {
        Task { @MainActor in
            do {
                let url = try await fileRepo.downloadFile(fromURL: fileURL)
                completion(url)
            } catch {
                Log.add(info: "Download File Error: \(error.localizedDescription)")
            }
        }
    }
    
}

extension AmityFileData {
    
    var fileName: String {
        return self.attributes["name"] as? String ?? ""
    }
}
