//
//  AmityChatFileViewCell.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 10/11/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

final class AmityChatFileViewCell: UITableViewCell, AmityChatCellProtocol {
    
    @IBOutlet weak var myImageView: UIImageView!
    @IBOutlet weak var displayNameLabel: UILabel!
    @IBOutlet weak var timeLabel: UILabel!
    @IBOutlet weak var messageStatusImageView: UIImageView!
    @IBOutlet weak var containerView: UIView!
    
    private var messageId: String?
    
    override func awakeFromNib() {
        super.awakeFromNib()
        messageStatusImageView.accessibilityIdentifier = "chat.message-cell.statusImageView"
        myImageView.backgroundColor = UIColor.systemGroupedBackground
        myImageView.contentMode = .scaleAspectFit
        
        containerView.backgroundColor = UIColor.systemGroupedBackground
        containerView.addRoundedCorner(radius: 8)
    }
    
    private func set(image: UIImage) {
        myImageView.image = image
    }
    
    func display(_ message: AmityMessage, client: AmityClient) {
        messageId = message.messageId
        
        if message.isDeleted {
            displayNameLabel.text = "Message deleted"
        } else {
            let flagCount = message.flagCount
            displayNameLabel.text = message.user?.displayName ?? "" + " F: \(flagCount)"
            messageStatusImageView.image = message.syncState.symbolImage()
            messageStatusImageView.accessibilityIdentifier = "chat.message-cell.statusImageView-\(message.syncState)"
            myImageView.image = UIImage(named:"file")

            setMetadata(for: message)
            
            if !message.isDeleted {
                let audioIcon = UIImage(systemName: "waveform")
                
                let iconConfiguration = UIImage.SymbolConfiguration(pointSize: 30, weight: .regular)
                let fileIcon = UIImage(systemName: "doc", withConfiguration: iconConfiguration)
                                
                self.myImageView.tintColor = UIColor.gray
                self.myImageView.preferredSymbolConfiguration = iconConfiguration
                self.myImageView.image = message.messageType == .audio ? audioIcon : fileIcon
                
                guard let fileInfo = message.getFileInfo() else { return }
                
                Log.add(info: "\nAudio Message:")
                Log.add(info: "Message Id: \(message.messageId)")
                Log.add(info: "File Id: \(fileInfo.fileId)")
                Log.add(info: "File URL: \(fileInfo.fileURL)")
            }
        }
    }
    
    
    private func setMetadata(for message: AmityMessage) {
        let messageCreateDate: NSAttributedString? = createDate(for: message)
        let messageTags: NSAttributedString? = tags(for: message)
        let childMessages: NSAttributedString? = self.childMessages(for: message)
        let hasParent: NSAttributedString? = parent(for: message)
        let readCount: NSAttributedString? = readCount(for: message)
        let deliveredCount: NSAttributedString? = deliveredCount(for: message)

        let mutableAttributedString = NSMutableAttributedString()

        let optionalAttributedStrings: [NSAttributedString?] = [
            hasParent,
            childMessages,
            messageTags,
            messageCreateDate,
            readCount,
            deliveredCount
        ]

        let attributedStrings: [NSAttributedString] = optionalAttributedStrings.compactMap { $0 }
        for attributedString in attributedStrings {
            mutableAttributedString.append(NSAttributedString(string: " "))
            mutableAttributedString.append(attributedString)
        }
        timeLabel.attributedText = mutableAttributedString
    }

    private func createDate(for message: AmityMessage) -> NSAttributedString {
        let createDateString: String = DateFormatter.localizedString(from: message.createdAt,
                                                                     dateStyle: .none,
                                                                     timeStyle: .short)
        return NSAttributedString(string: createDateString)
    }

    private func tags(for message: AmityMessage) -> NSAttributedString? {
        if let messageTags = message.tags as? [String],
            !messageTags.isEmpty {
            return attributedString(for: messageTags)
        }
        return nil
    }

    private func attributedString(for tags: [String]) -> NSAttributedString {
        assert(!tags.isEmpty)
        let colors: [UIColor] = ["PastelRed", "PastelOrange", "PastelYellow",
                                 "PastelGreen", "PastelBlue", "PastelPurple",
                                 "NeonPink", "NeonRed", "NeonGreen", "NeonAzure"].compactMap(UIColor.init)

        let mutableAttributedString = NSMutableAttributedString()
        for index in 0..<tags.count {
            let index = index % colors.count
            let attributedTagString = NSAttributedString(string: " \(tags[index]) ",
                attributes: [.backgroundColor: colors[index],
                             .foregroundColor: .white])
            mutableAttributedString.append(attributedTagString)
            mutableAttributedString.append(NSAttributedString(string: " "))
        }
        return mutableAttributedString
    }

    private func childMessages(for message: AmityMessage) -> NSAttributedString? {
        if message.childrenNumber > 0 {
            let attributedTagString = NSAttributedString(string: " childs: \(message.childrenNumber) ",
                attributes: [.backgroundColor: UIColor.black,
                             .foregroundColor: .white])
            return attributedTagString
        }
        return nil
    }

    private func parent(for message: AmityMessage) -> NSAttributedString? {
        if message.parentId != nil {
            return NSAttributedString(string: "🧒🏻")
        }
        return nil
    }
    
    private func readCount(for message: AmityMessage) -> NSAttributedString? {
        if message.readCount != 0 {
            return NSAttributedString(string: "R: \(message.readCount)")
        }
        return nil
    }
    
    private func deliveredCount(for message: AmityMessage) -> NSAttributedString? {
        if message.deliveredCount != 0 {
            return NSAttributedString(string: "D: \(message.deliveredCount)")
        }
        return nil
    }
    
    override func prepareForReuse() {
        super.prepareForReuse()
        myImageView.image = nil
        timeLabel.text = ""
        messageStatusImageView.image = nil
    }
}

extension AmityMessageType {
    
    var description: String {
        switch self {
        case .audio:
            return "Audio"
        case .image:
            return "Image"
        case .file:
            return "File"
        case .custom:
            return "Custom"
        case .text:
            return "Text"
        case .video:
            return "Video"
        @unknown default:
            return "Unknown"
        }
    }
}
