//
//  OnlineUsersViewController.swift
//  SampleApp
//
//  Created by Nishan on 17/7/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import UIKit

class OnlineUsersViewController: UIViewController {
    
    var tableView = UITableView()
    var onlineCountLabel = UILabel()
    let timestampLabel = UILabel()
    var manager = PresenceManager()
    var isPaginating: Bool = false
    
    let refreshControl = UIRefreshControl()
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupViews()
        
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: UITableViewCell.identifier)
        
        // Fetch user count
        fetchUserCount()
        
        fetchOnlineUsers()
        
        tableView.dataSource = self
        tableView.delegate = self
        
        tableView.reloadData()
        
        tableView.refreshControl = refreshControl
        
        refreshControl.addTarget(self, action: #selector(onRefreshTap), for: .valueChanged)
        
        let refreshIcon = UIBarButtonItem(image: UIImage(systemName: "arrow.2.squarepath"), style: .plain, target: self, action: #selector(onRefreshTap))
        self.navigationItem.rightBarButtonItem = refreshIcon
    }
    
    @objc func onRefreshTap() {
        fetchUserCount()
        
        fetchOnlineUsers()
    }
    
    func fetchUserCount() {
        Task {
            do {
                let count = try await manager.getOnlineUserCount()
                self.onlineCountLabel.text = "Online Users Count: \(count)"
                refreshControl.endRefreshing()
            } catch let error {
                AppUtility.showAlert(in: self, title: "", message: "Error while fetching online user count: \(error.localizedDescription)", action: nil)
                refreshControl.endRefreshing()
            }
        }
    }
    
    func fetchOnlineUsers() {
        Task {
            do {
                try await manager.getOnlineUsers()
                
                // Reload tableview
                tableView.reloadData()
                
                timestampLabel.text = "Fetched At: \(manager.snapshot?.fetchedAt?.description ?? "")"
            } catch let error {
                AppUtility.showAlert(in: self, title: "", message: "Error while fetching online user: \(error.localizedDescription)", action: nil)
            }
        }
    }
    
}

extension OnlineUsersViewController: UITableViewDataSource, UITableViewDelegate {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        guard let snapshot = manager.snapshot else { return 0 }
        
        return snapshot.users.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: UITableViewCell.identifier)!
        
        var config = cell.defaultContentConfiguration()
        
        if let snapshot = manager.snapshot {
            let user = snapshot.users[indexPath.row]
            config.text = user.displayName
            config.secondaryText = user.lastHeartbeat?.description ?? ""
        }
        
        cell.contentConfiguration = config
        
        return cell
    }
    
    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        let frameHeight = scrollView.frame.size.height
        let contentOffset = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height
        
        let distanceFromBottom = contentHeight - contentOffset
        if distanceFromBottom < frameHeight {
            paginateOnlineUsers()
        }
    }
    
    @MainActor
    private func paginateOnlineUsers() {
        guard let snapshot = manager.snapshot, !isPaginating else { return }
        
        if snapshot.canLoadMore {
            isPaginating = true
            Task{
                await snapshot.loadMore()
                
                tableView.reloadData()
                isPaginating = false
            }
        }
    }
}

extension OnlineUsersViewController {
    
    func setupViews() {
        let containerView = UIView()
        containerView.translatesAutoresizingMaskIntoConstraints = false
        containerView.backgroundColor = UIColor.systemGroupedBackground
        containerView.addRoundedCorner(radius: 12, cornerCurve: .circular)
        view.addSubview(containerView)
        
        let stackView = UIStackView()
        stackView.translatesAutoresizingMaskIntoConstraints = false
        stackView.axis = .vertical
        stackView.distribution = .fillEqually
        
        onlineCountLabel.text = "Online Users Count: \(100)"
        
        
        [onlineCountLabel, timestampLabel].forEach {
            $0.textColor = .black
            $0.numberOfLines = 1
            $0.font = UIFont.systemFont(ofSize: 15)
        }
        
        onlineCountLabel.translatesAutoresizingMaskIntoConstraints = false
        tableView.translatesAutoresizingMaskIntoConstraints = false
        
        containerView.addSubview(stackView)
        stackView.addArrangedSubview(onlineCountLabel)
        stackView.addArrangedSubview(timestampLabel)
        
        view.addSubview(tableView)
        
        NSLayoutConstraint.activate([
            
            containerView.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 8),
            containerView.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -8),
            containerView.topAnchor.constraint(equalTo: view.safeAreaLayoutGuide.topAnchor, constant: 8),
            containerView.heightAnchor.constraint(equalToConstant: 60),
            
            stackView.leadingAnchor.constraint(equalTo: containerView.leadingAnchor, constant: 12),
            stackView.trailingAnchor.constraint(equalTo: containerView.trailingAnchor, constant: -12),
            stackView.topAnchor.constraint(equalTo: containerView.topAnchor, constant: 4),
            stackView.bottomAnchor.constraint(equalTo: containerView.bottomAnchor, constant: -4),

            tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 0),
            tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: 0),
            tableView.bottomAnchor.constraint(equalTo: view.safeAreaLayoutGuide.bottomAnchor, constant: 0),
            tableView.topAnchor.constraint(equalTo: containerView.bottomAnchor, constant: 0),
        ])
    }
}
