//
//  PresenceSettingsViewController.swift
//  SampleApp
//
//  Created by Amity on 13/6/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import UIKit

class PresenceSettingsViewController: UIViewController {
    
    enum Element {
        case enable
        case disable
        case isEnabled
        case startHeartbeat
        case stopHeartbeat
        
        var title: String {
            switch self {
            case .enable:
                return "Enable Presence"
            case .disable:
                return "Disable Presence"
            case .isEnabled:
                return "Query Presence Settings"
            case .startHeartbeat:
                return "Start Heartbeat Sync"
            case .stopHeartbeat:
                return "Stop Heartbeat Sync"
            }
        }
    }
    
    enum ElementSection {
        case settings
        case heartbeat
        
        var title: String {
            switch self {
            case .settings:
                return "Presence Settings"
                
            case .heartbeat:
                return "Heartbeat"
            }
        }
        
        var elements: [Element] {
            switch self {
            case .heartbeat:
                return [.startHeartbeat, .stopHeartbeat]
            case .settings:
                return [.enable, .disable, .isEnabled]
            }
        }
    }
    
    let items: [ElementSection] = [.settings, .heartbeat]
    let tableView = UITableView(frame: .zero, style: .insetGrouped)
    let presenceManager = PresenceManager()
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        tableView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(tableView)
        NSLayoutConstraint.activate([
            view.leadingAnchor.constraint(equalTo: tableView.leadingAnchor),
            view.trailingAnchor.constraint(equalTo: tableView.trailingAnchor),
            view.safeAreaLayoutGuide.topAnchor.constraint(equalTo: tableView.safeAreaLayoutGuide.topAnchor),
            view.safeAreaLayoutGuide.bottomAnchor.constraint(equalTo: tableView.safeAreaLayoutGuide.bottomAnchor)
        ])
        
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: UITableViewCell.identifier)
        tableView.dataSource = self
        tableView.delegate = self
        
        self.title = "Presence Settings"
    }
}

extension PresenceSettingsViewController: UITableViewDataSource, UITableViewDelegate {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return items.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let section = items[section]
        return section.elements.count
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        let section = items[section]
        return section.title
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: UITableViewCell.identifier)!
        
        let element = items[indexPath.section].elements[indexPath.row]
        
        var content = cell.defaultContentConfiguration()
        content.text = element.title
        
        cell.contentConfiguration = content
        
        return cell
    }
    
    @MainActor
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let section = items[indexPath.section]
        let element = section.elements[indexPath.row]
        
        switch element {
        case .enable:
            Task {
                do {
                    try await presenceManager.togglePresence(enable: true)
                    self.showAlert(title: "", message: "User presence is enabled!")
                } catch let error {
                    self.showAlert(title: "", message: "Error while enabling user presence: \(error.localizedDescription)")
                }
            }
        case .disable:
            Task {
                do {
                    try await presenceManager.togglePresence(enable: false)
                    self.showAlert(title: "", message: "User presence is disabled!")
                } catch let error {
                    self.showAlert(title: "", message: "Error while disabling user presence: \(error.localizedDescription)")
                }
            }
        case .isEnabled:
            Task {
                do {
                    let isEnabled = try await presenceManager.fetchUserPresenceState()
                    self.showAlert(title: "", message: "User Presence Status: \(isEnabled ? "Enabled" : "Disabled")")
                } catch let error {
                    self.showAlert(title: "", message: "Error while fetching user presence status. \(error.localizedDescription)")
                }
            }
        case .startHeartbeat:
            Task {
                do {
                    try await presenceManager.startHeartbeat()
                    self.showAlert(title: "", message: "Heartbeat sync started!")
                } catch let error {
                    self.showAlert(title: "", message: "Error occurred while starting user presence heartbeat. \(error.localizedDescription)")
                }
            }
        case .stopHeartbeat:
            presenceManager.stopHeartbeat()
            self.showAlert(title: "", message: "Heartbeat sync stopped")
        }

    }
    
    func showAlert(title: String, message: String) {
        AppUtility.showAlert(in: self, title: title, message: message, action: nil)
    }
    
}
