//
//  RecordListViewController.swift
//  SampleAppLiveVideo
//
//  Created by Nutchaphon Rewik on 22/12/2563 BE.
//  Copyright © 2563 Amity. All rights reserved.
//

import UIKit
import AmitySDK
import AVKit

class RecordListViewController: UIViewController {
    
    var recordedStreamCollection: AmityCollection<AmityStream>?
    var token: AmityNotificationToken?
    
    @IBOutlet private weak var tableView: UITableView!
    @IBOutlet private weak var emptyView: UIView!
    
    deinit {
        token = nil
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        title = "Recorded Videos"
        tableView.register(LiveStreamCell.nib(), forCellReuseIdentifier: LiveStreamCell.identifier)
        setupDataSource()
        updateDataSource()
                
        NotificationCenter.default.addObserver(forName: Notification.Name.updateUser, object: nil, queue: nil) { [weak self] notification in

            self?.setupDataSource()
            self?.updateDataSource()
        }
        
    }
    
    private func presentPlaybackVideo(for recordingDataItem: AmityLiveVideoRecordingData) {
        guard let playBackUrl = recordingDataItem.url(for: .MP4) else {
            print("Unable to find playback url")
            return
        }
        let player = AVPlayer(url: playBackUrl)
        let playerController = AVPlayerViewController()
        playerController.player = player
        present(playerController, animated: true, completion: {
            player.play()
        })
    }
    
    private func presentListOfRecordedVideo(from stream: AmityStream) {
        
        let alertController = UIAlertController(title: "Recorded Video", message: nil, preferredStyle: .actionSheet)
        
        for (index, dataItem) in stream.recordingData.enumerated() {
            let title = "\(index + 1)"
            let playVideo = UIAlertAction(title: title, style: .default, handler: { [weak self] _action in
                self?.presentPlaybackVideo(for: dataItem)
            })
            alertController.addAction(playVideo)
        }
        
        let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancel)
        
        present(alertController, animated: true, completion: nil)
        
    }
    
    private func presentNoVideoRecordedDialogue() {
        let alertController = UIAlertController(title: "No recorded video", message: nil, preferredStyle: .alert)
        let ok = UIAlertAction(title: "OK", style: .cancel, handler: nil)
        alertController.addAction(ok)
        present(alertController, animated: true, completion: nil)
    }
    
    private func setupDataSource() {
        
        // Create query object.
        let query = AmityStreamCollectionQuery()
        query.includeStatus(.recorded)
        
        // Get colelction from the query that we craeted.
        recordedStreamCollection = AppManager.shared.streamRepository?.getStreams(from: query)
        token = recordedStreamCollection?.observe({ [weak self] collection, change, error in
            self?.updateDataSource()
        })
        
    }
    
    private func updateDataSource() {
        
        let hasDataToShow: Bool
        if let recordedStreamCollection = recordedStreamCollection, recordedStreamCollection.count() > 0 {
            hasDataToShow = true
        } else {
            hasDataToShow = false
        }
        emptyView.isHidden = hasDataToShow
        tableView.isHidden = !hasDataToShow
        
        tableView.reloadData()
    }
    
}

extension RecordListViewController: UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return Int(recordedStreamCollection?.count() ?? 0)
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        return tableView.dequeueReusableCell(withIdentifier: LiveStreamCell.identifier, for: indexPath)
    }
    
}

extension RecordListViewController: UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        
        let liveStreamCell = cell as! LiveStreamCell
        
        if let streamObject = recordedStreamCollection?.object(at: UInt(indexPath.row)) {
            liveStreamCell.configure(with: streamObject)
        } else {
            cell.textLabel?.text = "Not Found"
            cell.detailTextLabel?.text = "..."
        }
        
        // Load next page, when showing the last item in the collection.
        if let liveStreamCollection = recordedStreamCollection, indexPath.row == liveStreamCollection.count() - 1 {
            liveStreamCollection.nextPage()
        }
        
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return LiveStreamCell.height
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        tableView.deselectRow(at: indexPath, animated: true)
        
        if let stream = recordedStreamCollection?.object(at: UInt(indexPath.row)) {
            
            let recordingData = stream.recordingData
            if recordingData.count > 1 {
                presentListOfRecordedVideo(from: stream)
            } else if recordingData.count == 1 {
                presentPlaybackVideo(for: recordingData.first!)
            } else {
                presentNoVideoRecordedDialogue()
            }
            
        }
        
    }
    
}
